function [nn,Wn,Pn,nf,Wf,pf,cpu_time,tt,maxn,minn]=Brinkman2d_simplemodel(dx,T, bc,datas)
% 2d Brinkman tumor growth model FD-scheme
% without nutrient and drug, but source term depending on pressure
% model from a paper of Perthame, Vauchelet: Incompressible limit of mechanical model of tumor
%growth with viscosity.
%
% Input:    bc - boundary condition, can be 'N', 'P' or 'D' 
%           (homogeneous Neumann/periodic/Dirichlet),
%           T - run algorithm until that 'time'
%           dx - mesh width
% Output:   nn - number density of tumor cells at final time
%           nf - evolution of n, needed to produce movie
%           Wn - potential ($\nabla Wn$ is velocity)
%           Pn - pressure at final time
%           tt - vector with accumulated time steps
%           cpu_time - time needed for computation
%           maxn, minn - maximum and minimum of n over time at tt's
%               

% use mymovie.m for movies
% 13.11.2014
tic
if nargin <4
    D=[-3.0,3.0;-3.0,3.0]; 
    mynu=1;
    r=@(x,y)(sqrt(x.^2+y.^2));
    n0=@(x,y)(0.5*exp(-(r(x,y).^2)/0.1));
    
    % pressure law
    gamma1=10; % if you change gamma1 to a higher number it gets closer to
    % incompressible
    P=@(n)(n.^(gamma1));
    % definition of the source term (rhs of equation)
	a1=1; b1=1; G=@(p)((a1-b1*p)); % Pc=1;
    if nargin<3
        % set the boundary conditions ('P' = periodic, 'N'= Neumann,'D'
        % Dirichlet
%         bc='P'; %periodic
         bc='N'; % Neumann
        if nargin<2
            % set final time of simulation
            T=6;
            if nargin<1
                % mesh width
                dx=1/64;
            end
        end
    end
end
if nargin>=4 % if you input the parameters yourself as a struct
    D=datas.D; 
    mynu=datas.mynu;
    n0=datas.n0;
    gamma1=datas.gamma1;
    P=datas.P;
    G=datas.G;    
end

Gn=@(n)(G(P(n))); % source term function
% create the grid
xx=D(1,1)+dx/2:dx:D(1,2)-dx/2;
[X,Y]=meshgrid(xx);

% initialize density
n00=n0(X,Y);
n0=b_c(n00,bc);
curr_t=0;
nit=1;
maxn(1)=max(max(n0));
minn(1)=min(min(n0));
tt=0;

% just for movies
movierate=1; % how many time steps you want to store, if you make it too 
% big, your RAM might fill up
dts=movierate*dx/3;
NT=ceil(T/dts);
[Nx,Ny]=size(n00);
nf=zeros(Nx,Ny,NT);pf=zeros(Nx,Ny,NT);Wf=zeros(Nx,Ny,NT);
nf(:,:,1)=n0(2:end-1,2:end-1);
pf(:,:,1)=P(n0(2:end-1,2:end-1));
countt=2;
    
while curr_t<T-eps && nit<100000
   %   solve equation for potential (Brinkmann law)
   Wn= Helmholtz(P(n0(2:end-1,2:end-1)),mynu,dx,bc);
   umax=max(max(abs(Dx(Wn,dx))+abs(Dy(Wn,dx))')); % max velocity
   dt=min(dx/(2*(umax+0.1)), mynu/(gamma1*maxn(nit)^gamma1)); % CFL condition
   % evolution of cell density
   nn=evolvn(n0,Wn,Gn,dx,bc,dt); 
   n0=nn;
   nit= nit+1;
   curr_t=curr_t+dt;
   tt=[tt, curr_t];
   % maximum and minimum values of cell densities at current time
   maxn(nit)=max(max(n0));
   minn(nit)=min(min(n0));
   % save another time step for the space-time array
   if mod(nit,movierate)==0
        nf(:,:,countt)=n0(2:end-1,2:end-1);
        pf(:,:,countt)=P(n0(2:end-1,2:end-1));
        Wf(:,:,countt-1)=Wn(2:end-1,2:end-1);
        countt=countt+1;
   end    
end
Pn=P(nn); % pressure at final time
cpu_time = toc; % execution time

% plot the final values
figure;
surf(Wn(2:end-1,2:end-1)); colorbar
title(['W, ', 'gamma=',num2str(gamma1)]);
figure;surf(n0(2:end-1,2:end-1)); colorbar
title(['n, gamma=', num2str(gamma1), ', T=', num2str(curr_t), ...
    ', h=', num2str(dx)]);
end

% difference operators
function dd=Dx(v,dx) % in x-direction
    dd=diff(v,1,1)/dx;
end 

function dd=Dy(v,dx) % in y-direction
    dd=diff(v,1,2)/dx;
end 

function av=avx(v) % averaging operator in x direction
    av=(v(2:end,:)+v(1:end-1,:))/2;
end 

function av=avy(v) % averaging operator in y-direction
    av=(v(:,2:end)+v(:,1:end-1))/2;
end % 

% assign boundary conditions
function bc1=b_c(v, bc)
    if bc=='N' % Neumann
        bc1=[v(1,1),v(1,1:end),v(end,end); ...
            v(1:end,1),v,v(1:end,end); ...
            v(end,1),v(end,1:end), v(end,end)];  
    elseif bc=='P' % periodic
            bc1=[v(end,end),v(end,1:end),v(end,1); ...
                v(1:end,end),v,v(1:end,1); ...
                v(1,end),v(1,1:end), v(1,1)];
        elseif bc=='D' % Dirichlet
                    [~,N2]=size(v);
                    bc1=[zeros(1,N2+2); zeros(N2,1),v,zeros(N2,1); ...
                         zeros(1,N2+2)]; 
    end  
end 

% function to solve Brinkmann equation
function W=Helmholtz(p,mynu,dx,bc)
    [N1,N2]=size(p);
    Nbig=N1*N2;
    bigN=Nbig;
    p_vec=reshape(p,[Nbig, 1]); 
    Hh=mynu/dx^2;
    
    if bc=='N' % Neumann boundary conditions
        % set up matrix to be inverted
        v0=[1+2*Hh;(1+3*Hh)*ones(N1-2,1); 1+2*Hh];
        v1=[1+3*Hh;(1+4*Hh)*ones(N1-2,1); 1+3*Hh];
        v2=repmat(v1,[N2-2,1]);
        vec_c=[v0;v2;v0];
        vec_o=-Hh*ones(bigN,1);
        vi1=[0; -Hh*ones(N1-1,1)];
        vec_ir=repmat(vi1,[N2,1]);
        vec_il=[vec_ir(2:end,1);0];
        Helm=spdiags([vec_o vec_il vec_c vec_ir vec_o], ...
            [-N2 -1:1 N2],Nbig,Nbig);
        % solve the linear system
        W_vec=Helm\p_vec;
    elseif bc=='D' % Dirichlet boundary conditions, not sure this works 
            % set up the difference matrix for Dirichlet bc
            vec_c=(1+4*Hh)*ones(Nbig,1);
            vec_o=-Hh*ones(bigN,1);
            vi1=[0; -Hh*ones(N1-1,1)];
            vec_ir=repmat(vi1,[N2,1]);
            vec_il=[vec_ir(2:end,1);0];
            Helm=spdiags([vec_o vec_il vec_c vec_ir vec_o], ...
                [-N2 -1:1 N2],Nbig,Nbig);
            % solve resulting linear system
            W_vec=Helm\p_vec;
        elseif bc=='P' % periodic boundary conditions
                % set up matrix with differences for periodic bc
                vec_c=(1+4*Hh)*ones(Nbig,1);
                vec2=-Hh*ones(Nbig,1);
                Helm1=spdiags([vec2 vec2 vec_c vec2 vec2], ...
                    [-N2 -1:1 N2],Nbig,Nbig);
                Helm1(Nbig,1)=-Hh;
                Helm1(1,Nbig)=-Hh;
                % solve the linear system
                W_vec=Helm1\p_vec;
    end
    W=reshape(W_vec,[N1 N2]);    
    W=b_c(W,bc);  
end

% this function computes the evolution of the cell density with the upwind
% flux "upwind(n,W,dx)"
function nnew=evolvn(n,W,G,dx,bc,dt)
    nnewt=n(2:end-1,2:end-1)+dt*upwind(n,W,dx) ...
        +dt*sourcet(n(2:end-1,2:end-1),G);
    nnew=b_c(nnewt, bc);
end

% function to compute the source term, input density n and a fcn handle G
% for the source term
function ss=sourcet(n,G)
    ss=n.*G(n);
end 

% function to compute the upwind flux, input cell density n and potential W
% as well as grid size
function flux=upwind(n,W,dx)
    v=Dx(W(:,2:end-1),dx);
    u=Dy(W(2:end-1,:),dx);
    flux=Dx(v.*avx(n(:,2:end-1)),dx)+Dy(u.*avy(n(2:end-1,:)),dx) ...
        +0.5*dx*( Dx(abs(v).*Dx(n(:,2:end-1),dx),dx) ...
        + Dy(abs(u).*Dy(n(2:end-1,:),dx),dx));
end