function [output,cpu_time,tt,maxn,minn]=Brinkmannutrientdrug2d_implicit(dx,T, bc,bcdrug, bcnut, datas)
% 2d Brinkman tumor growth model FD-scheme
% with nutrient and drug, semi-implicit discretization of equations for q and c
% NOTE: Creating the spsdiags matrices seems very slow and consuming most
% of the execution time. It might be better to change this to normal
% matrices, but then the backslash operator might be slower.
%
% created 10.10.2016, modified 15.05.2017
%
% Input:    bc - boundary condition, can be 'N', 'P' or 'D' 
%           (homogeneous Neumann/periodic/Dirichlet), for potential and
%           cell density
%           T - run algorithm until that 'time'
%           dx - mesh width
%           bcdrug - boundary condition for drug, @function_handle(time),
%           only Dirichlet boundary conditions implemented!
%           bcnut - boundary condition for nutrient @function_handle(time),
%           only Dirichlet boundary conditions implemented!
%
% Output:   output- struct containing:
%           nn - number density of tumor cells at final time
%           nf - evolution of n, needed to produce movie
%           Wn - potential ($\nabla Wn$ is velocity)
%           Pn - pressure at final time
%           pf - pressure evolution over time, for movie
%           Wf - potential evolution over time; for movie
%           qn - drug at final time
%           cn - nutrient at final time
%           qf - evolution of drug
%           cf - evolution of nutrient
%           ______________________________________________________
%           tt - vector with accumulated time steps
%           cpu_time - time needed for computation
%           maxn, minn - maximum and minimum of n over time at tt's
%               
%
% use surf_movie.m for 3-d movies, movie_2d.m for 2d movies
% or movie_2d_sc.m for skin colored 2d-movies

tic
if nargin <6
    D=[-2.0,2.0;-2.0,2.0]; 
    mynu=.1;
%   rsquare = @(x,y)(4*((x).^2/1.25+ 2*(y).^2)); % causes instabilities
    rsquare = @(x,y)(2*((x-y).^2+ 4*(x+y).^2)); % causes some sort of instability
    % but may be related to the boundary
    % some initial data
    n0 =  @(x,y)(0.5*exp(-rsquare(x,y)/0.5));    
    nuc=5; % diffusion coefficients for nutrient and drug
    nuq=5;  
    % Pressure Law    
    gamma1=30;
    P=@(n)(n.^(gamma1));
    % Definition of Psi and Phi 
    a1=1; b1=1; a2=200;a3=200;a5=200;
    phi_=@(n,c,q,cb,qb)(a5*(a1-b1*P(n)).*(c-cb).*(c>cb)-a2*(cb-c).*(c<cb));
    psic_=@(n,c,cb)(-20*n.*c);
    psiq_=@(n,q,qb)(-30*n.*q);
    if nargin<5
        cbar=0.8; % boundary value for nutrient
        qbar=0; % boundary value for drug
        r_c = .0001; % rate of nutrient consumption of healthy cells
        r_q = 0.001; % rate of drug consumption
        ccrit=cbar/2; % critical nutrient concentration below which cells die
        qcrit=qbar/3;
       
        % homogeneous boundary conditions for drug and nutrient
        bcdrug=@(t)(qbar*ones(length(t),1));
        bcnut=@(t)(cbar*ones(length(t),1));        
        if nargin<3
            bc='N'; % Neumann
            if nargin<2
                T=1.5;
                if nargin<1
                    dx=1/64;
                end
            end                        
        end
    end
end

if nargin>=4 % if you do input the parameters
    D=datas.D; 
    mynu=datas.mynu;
    n0=datas.n0;
    gamma1=datas.gamma1;
    P=datas.P;
    nuc=datas.nuc;
    nuq=datas.nuq;
    phi_=datas.phi_;
    psic_=datas.psic_;
    psiq_=datas.psiq_;
    bcnut=datas.bcnut;
    bcdrug=datas.bcdrug;
    ccrit=datas.ccrit;
    qcrit=datas.qcrit;
end

% define source term functions based on ccrit, qcrit, critical
% concentrations
phin=@(n,c,q)(phi_(n,c,q,ccrit,qcrit));
psic=@(n,c)(psic_(n,c,ccrit));
psiq=@(n,q)(psiq_(n,q,qcrit));
% set up the spatial grid
xx=D(1,1)+dx/2:dx:D(1,2)-dx/2;
[X,Y]=meshgrid(xx);
X=X';
Y=Y'; 
xx_=D(1,1)-dx/2:dx:D(1,2)+dx/2;
[X_,Y_]=meshgrid(xx_);
X_=X_';
Y_=Y_';
% set up initial data
n00=n0(X,Y); % cell density
n0=b_c(n00,bc);
csupp=cbar;
qsupp = qbar;
q_0=@(x,y)(zeros(size(x,1),size(y,1))); % define fcn handle for drug initial data
c_0 = @(x,y)(cbar*ones(size(x,1),size(y,1))); % define function handle for nutrient input data
q0=q_0(X_,Y_); % set up drug initial data
q0_b= q_0(X_,Y_); % boundary conditions
gxq = [q0_b(2:end-1,1),q0_b(2:end-1,2)]; % extract the initial boundary conditions
gyq = [q0_b(1,2:end-1);q0_b(2,2:end-1)];
c0=c_0(X_,Y_); % set up nutrient initial data
c0_b = c_0(X_,Y_); % boundary conditions
gx = [c0_b(2:end-1,1),c0_b(2:end-1,2)]; % extract initial boundary conditions
gy = [c0_b(1,2:end-1);c0_b(2,2:end-1)];

curr_t=0;   % variable tracking the time at which we are computing
nit=1;      % number of iterations
maxn(1)=max(max(n0)); % variable tracking maximum of cell density
minn(1)=min(min(n0)); % variable tracking minimum of cell density
tt=0;                 % initial time, gonna be a vector containing all the
% discrete times at which a solution was computed

% just for movies/ some variables to store various time steps
moviefreq = 0.02; % time interval for which approximation is saved
Tm = 0:moviefreq:T;
movierate = 5;
tmovie = [];
dts = movierate*dx;
NT = ceil(T/dts);
[Nx,Ny]=size(n00);
nf=zeros(Nx,Ny,NT);
pf=zeros(Nx,Ny,NT);
Wf=zeros(Nx,Ny,NT);
cf=zeros(Nx,Ny,NT);
qf=zeros(Nx,Ny,NT);
nf(:,:,1)=n0(2:end-1,2:end-1);
pf(:,:,1)=P(n0(2:end-1,2:end-1));
qf(:,:,1)=q0(2:end-1,2:end-1);
cf(:,:,1)=c0(2:end-1,2:end-1);
countt=2;   % variable to count number of time steps saved
  
while curr_t<T-eps && nit<100000
    % solve Brinkman equation
    Wn = Helmholtz(P(n0(2:end-1,2:end-1)),mynu,dx,bc);
    phi_curr = phin(n0,c0,q0);
    
    % define CFL conditions
    phimax = max(max(abs(phi_curr)));
    rhs_c_max = max(max(abs(psic(n0,c0)-r_c)));
    rhs_q_max = max(max(abs(psiq(n0,q0)-r_q)));
    umax = max(max(abs(Dx(Wn,dx))+abs(Dy(Wn,dx))')); % maximum velocity
    dt = min(dx/(2*(umax+0.5+2.5*dx*phimax)), mynu/(gamma1*maxn(nit)^gamma1));
    dt = min(dt, min(1/rhs_c_max,1/rhs_q_max));
   
   % evolve n
   nn = evolvn(n0,Wn,phi_curr,dx,bc,dt);
   
   % evolve c and q
   Adiff_c = diffusion_matrix(Nx,nuc,dt,dx);
   rhs_c = rhs_diffusion(c0,n0,psic,csupp,r_c,nuc,gx,gy,dt,dx);
   cn = evolve_implicit_diffusion(rhs_c, Adiff_c,gx,gy);
   Adiff_q = diffusion_matrix(Nx,nuq,dt,dx);
   rhs_q = rhs_diffusion(q0,n0,psiq,qsupp,r_q,nuq,gxq,gyq,dt,dx);
   qn = evolve_implicit_diffusion(rhs_q,Adiff_q,gxq,gyq);

   % update variables
   n0=nn;
   c0=cn;
   q0=qn;
   nit= nit+1; % update number of iterations
   curr_t=curr_t+dt; % update current discrete time of evolution
   tt=[tt, curr_t]; % add new value to the vector with the time step values
   maxn(nit)=max(max(n0));
   minn(nit)=min(min(n0));
   % for movie variables
   if min(abs(Tm-curr_t))<dt
        nf(:,:,countt)=n0(2:end-1,2:end-1);
        pf(:,:,countt)=P(n0(2:end-1,2:end-1));
        Wf(:,:,countt-1)=Wn(2:end-1,2:end-1);
        cf(:,:,countt)=cn(2:end-1,2:end-1);
        qf(:,:,countt)=q0(2:end-1,2:end-1);
        countt=countt+1;
        tmovie=[tmovie,curr_t];      
   end    
   
   % next loop prints out simulation time and time step size every 250
   % timestep, so one can check whether everything is going well and the
   % time step size does not become too small
   if mod(nit,250)==0
       iteration_time= tt(end)
       datetime
       fprintf('current timestep size')
       dt   
   end
end

Pn=P(nn); % final pressure
al=max(max(nf,[],1),[],2); % nonzero part of nf vector
As=find(al); % finds all the nonzero entries

% create struct with output variables
output=struct('nn',nn,'nf',nf(:,:,As),'Wn',Wn,'Wf',Wf(:,:,As(1:end-1)), ...
     'Pn',Pn, 'pf', pf(:,:,As), 'qn',q0,'qf', ...
     qf(:,:,As),'cn',cn,'cf',cf(:,:,As),'D',D, 'tmovie',tmovie, ...
     'today',datetime);
 
cpu_time=toc; % final simulation time

end

%% subfunctions ==========================================================

% difference and averaging operators
function dd=Dx(v,dx) % difference in x-direction
    dd=diff(v,1,1)/dx;
end 

function dd=Dy(v,dx) % difference in y-direction
    dd=diff(v,1,2)/dx;
end 

function dd=Laplaz(v,dx) % discrete Laplace operator
    dd=1/dx^2*(v(3:end,2:end-1)+v(1:end-2,2:end-1)+v(2:end-1,3:end) ...
        +v(2:end-1,1:end-2)-4*v(2:end-1,2:end-1));
end

function av=avx(v) %averaging operator in x-direction
    av=(v(2:end,:)+v(1:end-1,:))/2;
end 

function av=avy(v) % averaging operator in y-direction
    av=(v(:,2:end)+v(:,1:end-1))/2;
end 

% function to assign boundary conditions
function bc1=b_c(v, bc, bc_value) 
    if nargin<3
        bc_value=0;
    end
    if bc=='N'
        bc1=[v(1,1),v(1,1:end),v(end,end); ...
            v(1:end,1),v,v(1:end,end); ...
            v(end,1),v(end,1:end), v(end,end)];  
    elseif bc=='P'
            bc1=[v(end,end),v(end,1:end),v(end,1); ...
                v(1:end,end),v,v(1:end,1); ...
                v(1,end),v(1,1:end), v(1,1)];
    elseif bc=='D'
                    [N1,N2]=size(v);
                    bc1=[bc_value*ones(1,N2+2); bc_value*ones(N1,1), v,...
                        bc_value*ones(N1,1);bc_value*ones(1,N2+2)];
    end
end 

% function to assign nonhomogeneous Dirichlet boundary conditions, for
% nutrient and drug
function bc2=b_c_D(g0x,g0y,v)
    % This is for nonhomogeneous Dirichlet boundary conditions
    % v are the values of the function inside the domain,
    % g0X is a dim(x-direction) x 2 vector containing the boundary data in 
    % in the x-direction, g0Y is a 2 x dim(y-direction) matrix containing
    % the boundary data in the y-direction
    
    bc2= [0,g0y(1,:),0; g0x(:,1), v, g0x(:,2);0,g0y(2,:),0];
end
%% "evolution" of W
function W=Helmholtz(p,mynu,dx,bc)
% solves the Helmholtz equation
    [N1,N2]=size(p);
    Nbig=N1*N2;
    p_vec=reshape(p,[Nbig, 1]); % matrix with pressure value is cast into a N*N vector
    Hh=mynu/dx^2; % coefficients in diffusion matrix
    
    if bc=='N' % Neumann
        % set up the difference matrix -mynu\Delta_h +Id
        v0=[1+2*Hh;(1+3*Hh)*ones(N1-2,1); 1+2*Hh];
        v1=[1+3*Hh;(1+4*Hh)*ones(N1-2,1); 1+3*Hh];
        v2=repmat(v1,[N2-2,1]);
        vec_c=[v0;v2;v0];
        vec_o=-Hh*ones(Nbig,1);
        vi1=[0; -Hh*ones(N1-1,1)];
        vec_ir=repmat(vi1,[N2,1]);
        vec_il=[vec_ir(2:end,1);0];
        Helm=spdiags([vec_o vec_il vec_c vec_ir vec_o], ...
            [-N2 -1:1 N2],Nbig,Nbig);
        % solve the linear system -mynu*\Delta_h W+W = p
        W_vec=Helm\p_vec;
    elseif bc=='D' 
            % set up the difference matrix -mynu\Delta_h +Id
            vec_c=(1+4*Hh)*ones(Nbig,1);
            vec_o=-Hh*ones(Nbig,1);
            vi1=[0; -Hh*ones(N1-1,1)];
            vec_ir=repmat(vi1,[N2,1]);
            vec_il=[vec_ir(2:end,1);0];
            Helm=spdiags([vec_o vec_il vec_c vec_ir vec_o], ...
                [-N2 -1:1 N2],Nbig,Nbig);
            % solve the linear system -mynu*\Delta_h W+W = p
            W_vec=Helm\p_vec;
    elseif bc=='P' 
                % set up the difference matrix -mynu\Delta_h +Id
                vec_c=(1+4*Hh)*ones(Nbig,1);
                vec2=-Hh*ones(Nbig,1);
                Helm1=spdiags([vec2 vec2 vec_c vec2 vec2], ...
                    [-N2 -1:1 N2],Nbig,Nbig);
                Helm1(Nbig,1)=-Hh;
                Helm1(1,Nbig)=-Hh;
                % solve the linear system -mynu*\Delta_h W+W = p
                W_vec=Helm1\p_vec;
    end
    % cast W-vector into a matrix with the values of the potential W     
    W=reshape(W_vec,[N1 N2]);    
    W=b_c(W,bc);  % assign boundary conditions
end
   
%% evolution of n, c, q   

% function to evolve cell density using upwind scheme
function nnew=evolvn(n,W,phi_n,dx,bc,dt) % evolution of cell density n
    nnewt = n(2:end-1,2:end-1) + dt*upwind(n,W,dx) ...
        + dt*sourcet(n(2:end-1,2:end-1),phi_n(2:end-1,2:end-1));
    nnew = b_c(nnewt, bc);
end

% sets up the matrix which is inverted in each step of the implicit schemes
% for c and q
function Adiff = diffusion_matrix(N,nu,dt,h)
% N is the number of grid cells in one dimension without boundary
% This gives the (sparse) matrix which should be inverted in each step of the
% implicit scheme for the diffusion equations
    Nbig = N^2;
    Hh = nu*dt/h^2;
    center_diag = (1+ 4*Hh)*ones(Nbig,1);
    off_diag_prime = repmat([-Hh*ones(N-1,1);0],N,1); % this might be inefficient but the function should only be run twice
    off_diag_up  = [0;off_diag_prime(1:end-1)];
    off_diag_down = off_diag_prime;
    far_off = -Hh*ones(Nbig,1);
    Adiff=spdiags([far_off off_diag_down center_diag off_diag_up far_off], ...
        [-N -1:1 N],Nbig,Nbig);

end

% implicit scheme for nutrient/drug equation
function cnew = evolve_implicit_diffusion(rhs, Adiff,g0x,g0y)
% g0x, g0y should be bc at next time step!
    [N1,N2]=size(rhs);
    Nbig=N1*N2;
    rhs_vec=reshape(rhs,[Nbig, 1]);
    c_vec = Adiff\rhs_vec;
    cnew = reshape(c_vec,[N1 N2]);    
    cnew = b_c_D(g0x,g0y,cnew);
end

% sets up the right hand side vector for the linear system which is
% solved in each step of the implicit scheme for the drug and the nutrient
function rhs = rhs_diffusion(cn,n,psi_c,c_supp,r_c,nu_c,g0x,g0y,dt,h)
    % right hand side for implicit scheme evolution of c, returns
    % N*N matrix (i.e. excluding boundary condition) whereas input n and cn
    % are with boundary conditions, gx, gy is boundary data for cn+1 (i.e.
    % at next timestep!
    % evolution eq for q is identical, so use this fcn for q eq as well
 
    rhs = cn(2:end-1,2:end-1).* ... 
        (1+dt * (psi_c(n(2:end-1,2:end-1),cn(2:end-1,2:end-1))-r_c)) ...
        + dt*r_c*c_supp;
    rhs([1 end],:) = rhs([1 end],:) + dt*nu_c/h^2*g0y; % makes sense
    rhs(:,[1 end]) = rhs(:,[1 end]) + dt*nu_c/h^2*g0x; % TEST THIS!!!
       
end

% source term setup
function ss=sourcet(n,G) % source term evalutation
% both inputs "matrices"
    ss=n.*G;
end

% upwind flux
function flux=upwind(n,W,dx) % flux in cell density equation, upwinding
    v=Dx(W(:,2:end-1),dx);
    u=Dy(W(2:end-1,:),dx);
    flux=Dx(v.*avx(n(:,2:end-1)),dx)+Dy(u.*avy(n(2:end-1,:)),dx) ...
        +0.5*dx*( Dx(abs(v).*Dx(n(:,2:end-1),dx),dx) ...
        + Dy(abs(u).*Dy(n(2:end-1,:),dx),dx));
end 