function [output,cpu_time,tt]=tumorqc_specialcase(dx,T, qbar, cbar,qcrit, ccrit, gamma1)
% model for tumor growth with Brinkman's law for velocity
% based on function Brinkmannutrientdrug2d_implicit.m, more specific input 
% data to test different parameters
%
% created 04.05.2015, modified 15.05.2017
%
% Input:    gamma1 - exponent in pressure law
%           ccrit  - "critical" concentration for nutrient
%           qcrit  - "critical" concentration for drug
%           cbar   - (constant in time) boundary condition for nutrient
%           qbar   - (constant in time) boundary condition for drug
%           T      - final computing time
%           dx     - mesh size
% 

% Output:   output- struct containing:
%           nn - number density of tumor cells at final time
%           nf - evolution of n, needed to produce movie
%           Wn - potential ($\nabla Wn$ is velocity)
%           Pn - pressure at final time
%           pf - pressure evolution over time, for movie
%           Wf - potential evolution over time; for movie
%           qn - drug at final time
%           cn - nutrient at final time
%           qf - evolution of drug
%           cf - evolution of nutrient
%           ______________________________________________________
%           tt - vector with accumulated time steps
%           cpu_time - time needed for computation
               

% use movie_2d.m or surf_movie.m for movies


if nargin<7
    gamma1=10;
    if nargin<6
        ccrit=0.1;
        if nargin<5 
            qcrit=0.1;
            if nargin<4
                cbar=0.5;
                if nargin<3
                    qbar=0.5;
                    if nargin<2
                        T=1;
                        if nargin<1
                            dx=1/16;
                        end
                    end
                end
            end
        end
    end
end

% function handle for the pressure law
P=@(n)(n.^(gamma1));
% create the source terms
a1=1; b1=1; a2=5; a3=3;
phi_=@(n,c,q,cb,qb)(a3*(a1-b1*P(n)).*(c-cb)-a2*(q-qb).*(q>qb));
psic_=@(n,c,cb)(-n.*c);
psiq_=@(n,q,qb)(-1*n.*q);

D=[-3.0,3.0;-3.0,3.0]; % domain
mynu=0.5; % diffusion coefficient in Brinkmann law
 
r=@(x,y)(sqrt(x.^2+y.^2));
n0=@(x,y)(0.5*exp(-((r(x,y).^2)/0.1))); % initial data for cell density
nuc=0.3; % diffusion coefficients for nutrient and drug
nuq=0.3;

% boundary conditions for drug and nutrient are constants
bcdrug=@(t)(qbar*ones(length(t),1));
bcnut=@(t)(cbar*ones(length(t),1));

% create a struct with parameters
datas=struct('D', D, 'n0', n0, 'mynu', mynu, 'bcdrug', bcdrug, 'bcnut', ...
    bcnut, 'nuc', nuc, 'nuq', nuq, 'P', P, 'ccrit', ccrit, 'qcrit', ... 
    qcrit, 'phi_', phi_, 'psic_', psic_, 'psiq_', psiq_, 'gamma1', gamma1 );


bc='N'; % boundary conditions

% run the simulation
[output,cpu_time,tt,~,~]=Brinkmannutrientdrug2d_implicit(dx,T,bc,bcdrug,bcnut,datas);
